/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* listserver processor.. this runs silently
*
**************************************************************************/
#include "defs.h"
#include "lib.p"

typedef struct
{
  char key[STR_SMALL + 1];
  char addr[STR_SMALL + 1];
} ls_t;

PRIVATE treap_t *ls_bt;

/**************************************************************************
* these routines were recycled from my scm project !!!
**************************************************************************/
/*
 * str must be free of spaces starting and ending.
 */
PRIVATE int valid_email(char *str)
{
  char *s;
  int  b = 0, c = 0;

  s = str;
  for(;;)
  {
    if(*s == 0)
    {
      if(isspace(*s))
        return 0;

      if(s > str)
      {
        if(*(s - 1) == '.')
          return 0;
      }
      else
        return 0;

      break;
    }

    if(*s == '.')
      b = 1;

    if(*s == '@')
    {
      if(c == 0)
        c = 1;
      else
        return 0;
    }

    s++;
  }

  if((b > 0) && (c == 1))
    return 1;

  return 0;
}

PRIVATE int extract_email_from_field(char *str, char **dstp)
{
  static char name[STR_LEN + 1], email[STR_LEN + 1];
  int    mode = 0, a = 0, b = 0;
  char *s;

  s = str;
  for(;;)
  {
    if(*s == 0)
      break;

    if(mode == 0)
    {
      if(*s == '<')
        mode = 2;
      else if(!isspace(*s))
        mode = 1;
      else
        s++;
    }
    else if(mode == 1)
    {
      if(isspace(*s))
        mode = 2;
      else
      {
        name[a] = *s;
        a++;
      }
      s++;
    }
    else if(mode == 2)
    {
      if(*s == '<')
        mode = 3;
      s++;
    }
    else if(mode == 3)
    {
      if(*s == '>')
        break;

      email[b] = *s;
      b++;
      s++;
    }
  }
  name[a] = 0;
  email[b] = 0;

  if(valid_email(email))
  {
    *dstp = email;
    return 1;
  }

  if(valid_email(name))
  {
    *dstp = name;
    return 1;
  }

  return 0;
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void load_conf(void)
{
  FILE *fp;
  ls_t entry;
  char str[STR_LEN + 1];

  int append_cmp_func(void *n1, void *n2)
  {
    return strcmp(((ls_t *)n1)->key, ((ls_t *)n2)->key);
  }

  if((fp = fopen(IPTRK_LS_CONF, "r")) == NULL)
    exit(-1);

  for(;;)
  {
    if(feof(fp))
      break;

    if(fgets(str, STR_LEN, fp) == NULL)
      break;

    if((str[0] == '#') || (str[0] < 32))
      continue;

    if(sscanf(str, "%s %s", entry.key, entry.addr) != 2)
      continue;

    treap_append(&ls_bt, &entry, sizeof(ls_t), append_cmp_func);
  }

  fclose(fp);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void post_stats(ls_t *e, char *to_str)
{
  FILE   *fp, *pp;
  char   str[STR_LEN + 1], *s;
  U_64   a, b, c, d, f;
  double aa, ab, ac, ad, af;

  if((fp = fopen("/tmp/iptrk_post", "w")) == NULL)
    exit(-1);

  fprintf(fp,
    "To: %s\n"
    "Subject: Current Stats for %s\n"
    "\n", to_str, e->addr);

  sprintf(str, INSTALL_PATH "/iptrkq -m 2 -s %s -e %s",
    e->addr, e->addr);
  pp = popen(str, "r");
  if(pp == NULL)
    exit(-1);

  for(;;)
  {
    if(feof(pp))
      break; 

    if(fgets(str, STR_LEN, pp) == NULL)
      break;  

    if(isdigit(str[0]))
    {
      s = str;
      if(!ignore_field(&s)) break;
      if(!get_next_vol_u64(&s, &a)) break;
      if(!get_next_vol_u64(&s, &b)) break;
      if(!get_next_vol_u64(&s, &c)) break;
      if(!get_next_vol_u64(&s, &d)) break;
      if(!get_next_vol_u64(&s, &f)) break;

      aa = (double)a / 1000000; /*a meg isn't 2 ^ 20, it's 10 ^ 6*/
      ab = (double)b / 1000000;
      ac = (double)c / 1000000;
      ad = (double)d / 1000000;
      af = (double)f / 1000000;

      fprintf(fp,
        "Your Volumes for this month are (in Mega Bytes)-\n\n"
        " External In: %.02f\n"
        "External Out: %.02f\n"
        "    Loopback: %.02f\n"
        " Internal In: %.02f\n"
        "Internal Out: %.02f\n"
        "\n"
        "--\niptrk R" REVISION, aa, ab, ac, ad, af);
      break;
    }
  }

  pclose(pp);
  fclose(fp);

  sprintf(str, "sendmail %s </tmp/iptrk_post &> /dev/null", to_str);
  system(str);
}

/**************************************************************************
*
**************************************************************************/
PRIVATE void read_mail(void)
{
  FILE *fp;
  char str[STR_LEN + 1],
       substr[STR_LEN + 1],
       *from_str = NULL,
       subject_str[STR_LEN + 1];
  int  state = 0;
  ls_t *p;

  int find_cmp_func(void *n)
  {
    return strcmp(subject_str, ((ls_t *)n)->key);
  }

  if((fp = fopen("/tmp/iptrk_mailbox", "r")) == NULL)  
    exit(-1);

  for(;;)
  {
    if(feof(fp))
      break;

    if(fgets(str, STR_LEN, fp) == NULL)
      break;

    if(str[0] == '\n' && str[1] == 0)
    {
      if(state == 0)
      {
        if(from_str != NULL &&  subject_str[0] != 0)
        {
          p = treap_find(&ls_bt, find_cmp_func);
          if(p != NULL)
            post_stats(p, from_str);
          from_str = NULL;
        }
      }
      state ^= 1;
    }

    if(get_param(substr, str, "From:", STR_LEN))
      extract_email_from_field(substr, &from_str);
    else
      get_param(subject_str, str, "Subject:", STR_LEN);
  }

  fclose(fp);
}

/**************************************************************************
*
**************************************************************************/
int main(void)
{
  if(system("mv " MAILBOX_PATH " /tmp/iptrk_mailbox &>/dev/null"))
    exit(-1);

  load_conf();
  read_mail();
  return 0;
}
